/*
 * Create and destroy user menus and all widgets in them. All widgets
 * are labels or pushbuttons; they are faster than text buttons. Whenever
 * the user presses a button with text in it, it is overlaid with a Text
 * button. For editing and input into the Text button, see useredit.c.
 *
 *	destroy_user_popup()		remove user popup
 *	create_user_popup()		create user popup
 */

#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifndef MIPS
#include <stdlib.h>
#endif
#include <pwd.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/LabelP.h>
#include <Xm/LabelG.h>
#include <Xm/PushBP.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleB.h>
#include <Xm/ScrolledW.h>
#include <Xm/Text.h>
#include <Xm/Protocols.h>
#ifdef DESKTOP
#include <Sgm/Finder.h>
#endif
#include "cal.h"

#define NCOLUMNS	6		/* # of widget columns in user list */

extern char *mystrdup();
extern void help_callback(), draw_calendar();
static void create_user_rows(), edit_user_button(), got_user_text(),
	    draw_row(), delete_callback(), sort_callback(), code_callback(),
	    done_callback(), list_callback(), got_text();
#ifdef MIPS
extern struct passwd *getpwnam();
#endif

extern Display		*display;	/* everybody uses the same server */
extern GC		gc;		/* everybody uses this context */
extern Pixel		color[NCOLS];	/* colors: COL_* */
extern Pixmap		pixmap[NPICS];	/* common symbols */
extern struct config	config;		/* global configuration data */
extern struct list	*mainlist;	/* list of all schedule entries */
extern struct mainmenu	mainmenu;	/* all important main window widgets */
extern struct user	*user;		/* user list (from file_r.c) */
extern int		nusers;		/* number of users in user list */

static BOOL		have_shell;	/* message popup exists if TRUE */
static Widget		shell;		/* popup menu shell */
static Widget		delete;		/* delete button, for desensitizing */
static Widget		info;		/* info line for error messages */
static Widget		textwidget;	/* if editing, text widget; else 0 */
static int		have_nrows;	/* # of table widget rows allocated */
static int		xedit, yedit;	/* if editing, column/row */
static int		ycurr;		/* current row, 0=none, 1=1st user...*/
static Widget		ulist;		/* user list RowColumn widget */
static Widget	(*utable)[NCOLUMNS+1];	/* all widgets in user list table */
					/* [0][*] is title row, */
					/* [*][NCOLUMNS] is form widget */



/*
 * destroy the popup. Remove it from the screen, and destroy its widgets.
 * Redraw the week menu if there is one.
 */

void destroy_user_popup()
{
	if (have_shell) {
		XtPopdown(shell);
		XTDESTROYWIDGET(shell);
		have_shell = FALSE;
		mainlist->modified = TRUE;
		write_mainlist();
		resynchronize_daemon();
		(void)read_mainlist(0);
		update_all_listmenus();
		draw_calendar(NULL);
		draw_week_calendar();
	}
}


/*
 * create a user popup as a separate application shell.
 */

void create_user_popup()
{
	Widget			form, scroll, w;
	Arg			args[15];
	int			n;
	Atom			closewindow;

	if (have_shell)
		return;
	(void)write_mainlist();	/* last chance to write modified foreign apts*/

	n = 0;
	XtSetArg(args[n], XmNdeleteResponse,	XmUNMAP);		n++;
	XtSetArg(args[n], XmNiconic,		False);			n++;
	shell = XtAppCreateShell("User List", "plan",
			applicationShellWidgetClass, display, args, n);
#	ifdef EDITRES
	XtAddEventHandler(shell, (EventMask)0, TRUE, 
 			_XEditResCheckMessages, NULL);
#	endif
	set_icon(shell, 1);
	form = XtCreateWidget("userform", xmFormWidgetClass,
			shell, NULL, 0);
	XtAddCallback(form, XmNhelpCallback, help_callback, (XtPointer)"user");

							/*-- buttons --*/
	n = 0;
	XtSetArg(args[n], XmNbottomAttachment,	XmATTACH_FORM);		n++;
	XtSetArg(args[n], XmNbottomOffset,	8);			n++;
	XtSetArg(args[n], XmNleftAttachment,	XmATTACH_FORM);		n++;
	XtSetArg(args[n], XmNleftOffset,	8);			n++;
	XtSetArg(args[n], XmNwidth,		80);			n++;
	XtSetArg(args[n], XmNsensitive,		False);			n++;
	delete = w = XtCreateManagedWidget("Delete", xmPushButtonWidgetClass,
			form, args, n);
	XtAddCallback(w, XmNactivateCallback, delete_callback, (XtPointer)0);
	XtAddCallback(w, XmNhelpCallback,     help_callback,   (XtPointer)
								"user_delete");

	n = 0;
	XtSetArg(args[n], XmNbottomAttachment,	XmATTACH_FORM);		n++;
	XtSetArg(args[n], XmNbottomOffset,	8);			n++;
	XtSetArg(args[n], XmNleftAttachment,	XmATTACH_WIDGET);	n++;
	XtSetArg(args[n], XmNleftWidget,	w);			n++;
	XtSetArg(args[n], XmNleftOffset,	8);			n++;
	XtSetArg(args[n], XmNwidth,		80);			n++;
	w = XtCreateManagedWidget("Sort", xmPushButtonWidgetClass,
			form, args, n);
	XtAddCallback(w, XmNactivateCallback, sort_callback, (XtPointer)0);
	XtAddCallback(w, XmNhelpCallback,     help_callback, (XtPointer)
								"user_sort");
	n = 0;
	XtSetArg(args[n], XmNbottomAttachment,	XmATTACH_FORM);		n++;
	XtSetArg(args[n], XmNbottomOffset,	8);			n++;
	XtSetArg(args[n], XmNrightAttachment,	XmATTACH_FORM);		n++;
	XtSetArg(args[n], XmNrightOffset,	8);			n++;
	XtSetArg(args[n], XmNwidth,		80);			n++;
	w = XtCreateManagedWidget("Done", xmPushButtonWidgetClass,
			form, args, n);
	XtAddCallback(w, XmNactivateCallback, done_callback, (XtPointer)0);
	XtAddCallback(w, XmNhelpCallback,     help_callback, (XtPointer)
								"user_done");

	n = 0;
	XtSetArg(args[n], XmNbottomAttachment,	XmATTACH_FORM);		n++;
	XtSetArg(args[n], XmNbottomOffset,	8);			n++;
	XtSetArg(args[n], XmNrightAttachment,	XmATTACH_WIDGET);	n++;
	XtSetArg(args[n], XmNrightWidget,	w);			n++;
	XtSetArg(args[n], XmNrightOffset,	8);			n++;
	XtSetArg(args[n], XmNwidth,		80);			n++;
	w = XtCreateManagedWidget("Help", xmPushButtonWidgetClass,
			form, args, n);
	XtAddCallback(w, XmNactivateCallback, help_callback, (XtPointer)
								"user");
	XtAddCallback(w, XmNhelpCallback,     help_callback, (XtPointer)
								"user");

							/*-- infotext -- */
	n = 0;
	XtSetArg(args[n], XmNleftAttachment,	XmATTACH_FORM);		n++;
	XtSetArg(args[n], XmNleftOffset,	8);			n++;
	XtSetArg(args[n], XmNrightAttachment,	XmATTACH_FORM);		n++;
	XtSetArg(args[n], XmNrightOffset,	8);			n++;
	XtSetArg(args[n], XmNbottomAttachment,	XmATTACH_WIDGET);	n++;
	XtSetArg(args[n], XmNbottomWidget,	w);			n++;
	XtSetArg(args[n], XmNbottomOffset,	8);			n++;
	XtSetArg(args[n], XmNalignment,         XmALIGNMENT_BEGINNING);	n++;
	info = XtCreateManagedWidget(" ", xmLabelGadgetClass,
			form, args, n);

							/*-- scroll --*/
	n = 0;
	XtSetArg(args[n], XmNtopAttachment,	XmATTACH_FORM);		n++;
	XtSetArg(args[n], XmNtopOffset,		8);			n++;
	XtSetArg(args[n], XmNbottomAttachment,	XmATTACH_WIDGET);	n++;
	XtSetArg(args[n], XmNbottomWidget,	info);			n++;
	XtSetArg(args[n], XmNbottomOffset,	8);			n++;
	XtSetArg(args[n], XmNleftAttachment,	XmATTACH_FORM);		n++;
	XtSetArg(args[n], XmNleftOffset,	8);			n++;
	XtSetArg(args[n], XmNrightAttachment,	XmATTACH_FORM);		n++;
	XtSetArg(args[n], XmNrightOffset,	8);			n++;
	XtSetArg(args[n], XmNwidth,		690);			n++;
	XtSetArg(args[n], XmNheight,		220);			n++;
	XtSetArg(args[n], XmNscrollingPolicy,	XmAUTOMATIC);		n++;
	scroll = XtCreateWidget("uscroll", xmScrolledWindowWidgetClass,
			form, args, n);
	XtAddCallback(scroll, XmNhelpCallback, help_callback, (XtPointer)
								"user");

	n = 0;
	ulist = XtCreateManagedWidget("ulist", xmBulletinBoardWidgetClass,
			scroll, args, n);

	create_user_rows();	/* have_shell must be FALSE here */

	XtManageChild(scroll);
	XtManageChild(form);
	XtPopup(shell, XtGrabNone);

	closewindow = XmInternAtom(display, "WM_DELETE_WINDOW", False);
	XmAddWMProtocolCallback(shell, closewindow,
					done_callback, (XtPointer)shell);
	have_shell = TRUE;
}


/*
 * makes sure there are enough widget rows for schedule entries. Also makes
 * sure that there aren't too many, for speed reasons. Allocate one extra
 * widget row for the title at the top. All the text buttons are
 * label widgets. For performance reasons, they are overlaid by a text
 * widget when pressed.
 * No text is printed into the buttons yet, this is done later by draw_users().
 */

static short cell_x    [NCOLUMNS] = {  4,  62, 114, 164, 224, 344 };
static short cell_xs   [NCOLUMNS] = { 50,  50,  50,  60, 120, 300 };
static char *cell_name [NCOLUMNS] = { "Month", "Week", "Alarm", "Group",
				      "Name", "Home or File"};
static char *cell_help [NCOLUMNS] = { "user_enable", "user_enable",
				      "user_enable", "user_color",
				      "user_name", "user_home" };

static void create_user_rows()
{
	int			nrows = nusers+5 - nusers%5;
	int			x, y;
	Arg			args[20];
	int			n;
	int			align;
	char			*name;
	WidgetClass		class;

	if (!have_shell)				/* check # of rows: */
		have_nrows = 0;
	if (nrows <= have_nrows)
		return;

	n = (nrows+1) * (NCOLUMNS+1) * sizeof(Widget *);
	if (utable && !(utable = (Widget (*)[])realloc(utable, n)) ||
	   !utable && !(utable = (Widget (*)[])malloc(n)))
		fatal("no memory");

	XtUnmanageChild(ulist);
	for (y=have_nrows; y <= nrows; y++) {
	    n = 0;
#ifdef noDESKTOP
	    if (y == 999) {
		XtSetArg(args[n], XmNtopAttachment,	XmATTACH_WIDGET); n++;
		XtSetArg(args[n], XmNtopWidget,	  utable[y-1][NCOLUMNS]); n++;
	    } else {
		XtSetArg(args[n], XmNtopAttachment,	XmATTACH_FORM);   n++;
		XtSetArg(args[n], XmNy,			10 + 40 * y);		  n++;
	    }
#else
	    XtSetArg(args[n], XmNtopAttachment,		XmATTACH_FORM);   n++;
	    XtSetArg(args[n], XmNheight,		30);		  n++;
	    XtSetArg(args[n], XmNy,			10 + 30*y);	  n++;
#endif
	    XtSetArg(args[n], XmNleftAttachment,	XmATTACH_FORM);   n++;
	    utable[y][NCOLUMNS] = XtCreateWidget("userrowform",
	   				 xmFormWidgetClass, ulist, args, n);
	    for (x=0; x < NCOLUMNS; x++) {
		n = 0;
		align = XmALIGNMENT_BEGINNING;
		name  = cell_name[x];
		class = xmPushButtonWidgetClass;
		if (y) {
#ifdef noDESKTOP
			if (x == 5)
				class = sgFinderWidgetClass;
#endif
			if (x < 3) {
				class = xmToggleButtonWidgetClass;
				XtSetArg(args[n], XmNselectColor,
						color[COL_TOGGLE]);	n++;
			}
			name  = " ";
			if (x == 3)
				align = XmALIGNMENT_CENTER;
		} else {
			class = xmLabelWidgetClass;
			align = XmALIGNMENT_CENTER;
		}
		XtSetArg(args[n], XmNx,			cell_x[x]);	n++;
		XtSetArg(args[n], XmNwidth,		cell_xs[x]);	n++;
		XtSetArg(args[n], XmNtopAttachment,	XmATTACH_FORM); n++;
		XtSetArg(args[n], XmNbottomAttachment,	XmATTACH_FORM);	n++;
		XtSetArg(args[n], XmNalignment,         align);		n++;
		XtSetArg(args[n], XmNrecomputeSize,	False);		n++;
		XtSetArg(args[n], XmNtraversalOn,	True);		n++;
		XtSetArg(args[n], XmNhighlightThickness,0);		n++;
		XtSetArg(args[n], XmNshadowThickness,	x > 2 && y);	n++;
		utable[y][x] = XtCreateManagedWidget(name, class,
				utable[y][NCOLUMNS], args, n);
		if (y)
			XtAddCallback(utable[y][x],
				x > 2 ? XmNactivateCallback
				      : XmNvalueChangedCallback,
				list_callback, (XtPointer)(x + y * NCOLUMNS));
		XtAddCallback(utable[y][x], XmNhelpCallback, help_callback,
						(XtPointer)cell_help[x]);
	    }
	    XtManageChild(utable[y][NCOLUMNS]);
	}
	for (y=have_nrows; y <= nrows; y++)
		draw_row(y);
	have_nrows = nrows;

	XtManageChild(ulist);
}


/*-------------------------------------------------- editing ----------------*/
/*
 * turn a text label into a Text button, to allow user input. This is done
 * by simply installing a text widget on top of the label widget. The proper
 * user name or home dir is put into the text widget. The previously edited
 * button is un-edited.
 */

static void edit_user_button(doedit, x, y)
	BOOL			doedit;		/* TRUE=edit, FALSE=unedit */
	int			x;		/* column, 0..NCOLUMNS-1* */
	int			y;		/* row, y=0: title */
{
	Arg			args[15];
	int			n;
	char			*text;

	if (textwidget) {
		char *string = XmTextGetString(textwidget);
		got_user_text(xedit, yedit, string);
		XtFree(string);
		XtDestroyWidget(textwidget);
		if (yedit && yedit <= nusers)
			user[yedit-1].suspend_m =
			user[yedit-1].suspend_w = 0;
		draw_row(yedit);
		create_user_rows();
	}
	textwidget = 0;
	if (!doedit)
		return;

	if (y > nusers+1)
		y = nusers+1;
	n = 0;
	XtSetArg(args[n], XmNx,			10 + cell_x[x]);	n++;
	XtSetArg(args[n], XmNy,			10 + 30*y);		n++;
	XtSetArg(args[n], XmNwidth,		cell_xs[x]);		n++;
	XtSetArg(args[n], XmNheight,		30);			n++;
	XtSetArg(args[n], XmNrecomputeSize,	False);			n++;
	XtSetArg(args[n], XmNpendingDelete,	True);			n++;
	XtSetArg(args[n], XmNhighlightThickness,0);			n++;
	XtSetArg(args[n], XmNshadowThickness,	1);			n++;
	XtSetArg(args[n], XmNbackground,	color[COL_TEXTBACK]);	n++;
	textwidget = XtCreateManagedWidget("text", xmTextWidgetClass,
			ulist, args, n);
	XtAddCallback(textwidget, XmNactivateCallback, got_text,
						(XtPointer)(x + y * NCOLUMNS));
	XtAddCallback(textwidget, XmNhelpCallback, help_callback,
						(XtPointer)cell_help[x]);
	XmProcessTraversal(textwidget, XmTRAVERSE_CURRENT);

	text = y > nusers ? "" : x == 4 ? user[y-1].name : user[y-1].home;
	print_text_button(textwidget, "%s", text);
	xedit = x;
	yedit = y;
}


/*
 * the user has entered text into a name or home text button. Check & store.
 */

static void got_user_text(x, y, string)
	int			x;		/* column, 0..NCOLUMNS-1* */
	int			y;		/* row, y=0: title */
	char			*string;	/* text entered by user */
{
	struct passwd		*pw;		/* for searching home dirs */
	register struct user	*u;
	char			buf[100];
	int			i;

	if (!y--)
		return;
	while (*string == ' ' || *string == '\t')
		string++;
	while ((i = strlen(string)) && string[i-1] == ' ')
		string[i-1] = 0;
	if (!*string)
		return;
	for (i=0; i < nusers; i++) {
		if (x == 4 && i != y && !strcmp(string, user[i].name)) {
			create_error_popup(shell, 0,
				"User name is not unique, rejected");
			return;
		}
		if (x == 5 && i != y && !strcmp(string, user[i].home)) {
			create_error_popup(shell, 0,
				"Home directory is not unique, rejected");
			return;
		}
	}
	if (y >= nusers) {
		int n = ++nusers * sizeof(struct user);
		if (user && !(user = (struct user *)realloc(user, n)) ||
		   !user && !(user = (struct user *)malloc(n)))
			fatal("no memory");
		y = nusers-1;
		u = user+y;
		u->name	     = 0;
		u->home	     = 0;
		u->suspend_m = 0;
		u->suspend_w = 0;
		u->alarm     = 0;
		u->color     = 0;
		u->time	     = 0;
	}
	u = user+y;
	if (x == 4) {					/* name */
		if (u->name)
			free(u->name);
		if (u->home)
			free(u->home);
		u->name = mystrdup(string);
		u->home = 0;
	} else {					/* home */
		if (u->home)
			free(u->home);
		u->home = mystrdup(string);
		if (*u->home && access(u->home, F_OK))
			print_button(info, "%s: %s", u->home,
				errno==ENOENT ? "new file" : "cannot access");
	}
	if (!*string)
		print_button(info, "Null user not allowed");

	if (u->name && *u->name && !u->home) {		/* default home? */
		if (pw = getpwnam(string))
			u->home = mystrdup(pw->pw_dir);
		else {
			sprintf(buf, "%s/%s", LIB, string);
			print_button(info, "no user \"%s\"", string);
			u->home = mystrdup(buf);
		}
	}
}


/*
 * draw all buttons of row y. y must be > 0 because 0 is the title row.
 * If y is > nusers, the row is blanked.
 */

static void draw_row(y)
	int			y;
{
	Arg			arg;
	register struct user	*u = &user[y-1];

	if (y < 1)
		return;
	if (y <= nusers) {					/* valid row */
		XtSetArg(arg, XmNset, !u->suspend_m);
		XtSetValues (utable[y][0], &arg, 1);
		XtSetArg(arg, XmNset, !u->suspend_w);
		XtSetValues (utable[y][1], &arg, 1);
		XtSetArg(arg, XmNset,  u->alarm);
		XtSetValues (utable[y][2], &arg, 1);
		XtSetArg(arg, XmNbackground, color[COL_WUSER_0 + u->color]);
		XtSetValues (utable[y][3], &arg, 1);
		print_button(utable[y][3], "%d", u->color);
		print_button(utable[y][4], "%s", u->name);
		print_button(utable[y][5], "%s", u->home);
	} else {						/* blank row */
		XtSetArg(arg, XmNset, 0);
		XtSetValues (utable[y][0], &arg, 1);
		XtSetValues (utable[y][1], &arg, 1);
		XtSetValues (utable[y][2], &arg, 1);
		XtSetArg(arg, XmNbackground, color[COL_BACK]);
		XtSetValues (utable[y][3], &arg, 1);
		print_button(utable[y][3], " ");
		print_button(utable[y][4], " ");
		print_button(utable[y][5], " ");
	}
}



/*-------------------------------------------------- callbacks --------------*/
/*
 * Delete, Add-all, Sort, and Done buttons
 * All of these routines are direct X callbacks.
 */

/*ARGSUSED*/
static void delete_callback(widget, item, data)
	Widget				widget;
	int				item;
	XmToggleButtonCallbackStruct	*data;
{
	int				n;
	Arg				args;

	if (ycurr && ycurr <= nusers) {
		edit_user_button(FALSE, 0, 0);
		for (n=ycurr-1; n < nusers; n++)
			user[n] = user[n+1];
		n = --nusers * sizeof(struct user);
		if (n && !(user = (struct user *)realloc(user, n)))
			fatal("no memory");
		for (n=1; n <= have_nrows; n++)
			draw_row(n);
	}
	if (!ycurr) {
		XtSetArg(args, XmNsensitive, 0);
		XtSetValues(delete, &args, 1);
	}
}


/* aren't prototypes annoying? :-) */
#if defined(ULTRIX) || defined(MIPS)
		/* this means Ultrix 4.2A. If Ultrix 4.3 complains about */
		/* a missing const, change the following definition. */
#define CONST
#else
#define CONST const
#endif

static int compare(u, v) register CONST void *u, *v; {
	return(  ((struct user *)u)->color == ((struct user *)v)->color
	? strcmp(((struct user *)u)->name,    ((struct user *)v)->name)
	:        ((struct user *)u)->color -  ((struct user *)v)->color); }

/*ARGSUSED*/
static void sort_callback(widget, item, data)
	Widget				widget;
	int				item;
	XmToggleButtonCallbackStruct	*data;
{
	Arg				args;
	int				n;

	if (nusers) {
		edit_user_button(FALSE, 0, 0);
		XtSetArg(args, XmNsensitive, 0);
		XtSetValues(delete, &args, 1);
		qsort(user, nusers, sizeof(struct user), compare);
		for (n=1; n <= nusers; n++)
			draw_row(n);
	}
}

/*ARGSUSED*/
static void code_callback(widget, item, data)
	Widget				widget;
	int				item;
	XmToggleButtonCallbackStruct	*data;
{
	config.colorcode_m ^= TRUE;
	draw_calendar(NULL);
}

/*ARGSUSED*/
static void done_callback(widget, item, data)
	Widget				widget;
	int				item;
	XmToggleButtonCallbackStruct	*data;
{
	edit_user_button(FALSE, 0, 0);
	destroy_user_popup();
	draw_calendar(NULL);
	draw_week_calendar();
}


/*
 * one of the buttons in the list was pressed
 */

/*ARGSUSED*/
static void list_callback(widget, item, data)
	Widget				widget;
	int				item;
	XmToggleButtonCallbackStruct	*data;
{
	int				x = item % NCOLUMNS;
	int				y = item / NCOLUMNS;
	Arg				arg;

	if (y > nusers) {					/* new entry */
		if (x != 4) {
			print_button(info, "Enter user name first");
			return;
		}
		ycurr = 0;
		edit_user_button(TRUE, x, nusers+1);
	} else {						/* old entry */
		ycurr = y;
		switch(x) {
		  case 0:					/* month */
			user[y-1].suspend_m = !data->set;
			draw_calendar(NULL);
			break;
		  case 1:					/* week */
			user[y-1].suspend_w = !data->set;
			draw_week_calendar(NULL);
			break;
		  case 2:					/* alarm */
			user[y-1].alarm     =  data->set;
			break;
		  case 3:					/* color */
			user[y-1].color++;
			user[y-1].color &= 7;
			XtSetArg(arg, XmNbackground,
				color[COL_WUSER_0 + user[y-1].color]);
			XtSetValues(widget, &arg, 1);
			print_button(widget, "%d", user[y-1].color);
			break;
		  case 4:					/* username */
		  case 5:					/* home dir */
			edit_user_button(TRUE, x, y);
		}
	}
	XtSetArg(arg, XmNsensitive, ycurr > 0);
	XtSetValues(delete, &arg, 1);
	print_button(info, " ");
}


/*
 * the user pressed Return in a text entry button
 */

/*ARGSUSED*/
static void got_text(widget, item, data)
	Widget				widget;
	int				item;
	XmToggleButtonCallbackStruct	*data;
{
	int				x = item % NCOLUMNS;
	int				y = item / NCOLUMNS;

	edit_user_button(FALSE, 0, 0);
	if (x == 4)
		edit_user_button(TRUE, 5, y);
}
