/*
 * draws the week view. PostScript printing is also here because it uses
 * the same high-level routines, only the low-level output is diverted.
 *
 *	clicked_week_calendar(x, y)	Press on the week display detected.
 *	draw_week_day(day, month, year)	The day day/month/year has changed,
 *					redraw the week view if it contains
 *					that day.
 *	draw_week_calendar()		If there is a week menu, resize, and
 *					redraw
 *	redraw_week_calendar()		Same, but assume that it's still the
 *					same week, so don't recalc and resize.
 *	print_week_calendar(fp,l)	print PostScript week to fp
 */

#include <stdio.h>
#include <time.h>
#include <Xm/Xm.h>
#include "cal.h"

#define DBLTIME		1000		/* doubleclick time [ms] */

#define SLOPE		1/4		/* slope of arrow head / barheight */
#define MINLEN		8		/* no bar is shorter than MINLEN */

#define TOD(t)		((t)%86400)
#define BOUND(t,a,b)	((t)<(a) ? (a) : (t)>(b) ? (b) : (t));
#define XPOS(t)		(((t) - c->week_minhour*3600) * c->week_hourwidth/3600)


extern char *mktimestring(), *mkdatestring(), *parse_holidays();
extern struct tm *time_to_tm();
extern time_t tm_to_time();
extern void build_week(), create_week_menu();
static void draw_week_day_foreground(), draw_week_day_background(), draw_bar();
void draw_week_calendar(), redraw_week_calendar();

extern Display		*display;	/* everybody uses the same server */
extern GC		jgc, gc;	/* graphic context, Japanese and std */
extern XFontStruct	*font[NFONTS];	/* fonts: FONT_* */
extern struct config	config;		/* global configuration data */
extern struct mainmenu	mainmenu;	/* all important main window widgets */
extern struct user	*user;		/* user list (from file_r.c) */
extern time_t		curr_week;	/* week being displayed, time in sec */
extern struct week	week;		/* info on week view */
extern struct list	*mainlist;	/* list of all schedule entries */
extern struct holiday	holiday[366];	/* info for each day, separate for */
extern struct holiday	sm_holiday[366];/* full-line texts under, and small */

extern char *weekday_name[];
extern char *monthname[];


/*
 * got a click in the calendar area. Traverse the node tree, first down until
 * the correct line is found, and then horizontally until the node is found.
 */

void clicked_week_calendar(xc, yc, time)
	int			xc, yc;		/* pixel pos clicked */
	Time			time;		/* current time in ms */
{
	register struct config	*c = &config;
	struct weeknode		*vert, *horz;	/* week node scan pointers */
	register struct entry	*ep;		/* entry to test */
	static struct entry	*last_ep;	/* last entry found (dblclk?)*/
	static Time		last_time;	/* time of last press */
	int			d;		/* day counter */
	int			yd;		/* y start coord of day box */
	int			x, y;		/* upper left of day bk box */
	int			ys;		/* height of bar line w/ gaps*/
	time_t			btime;		/* start time of bar */
	int			b, e;		/* X start end end pos of bar*/
	int			xend;		/* right margin of chart */
	int			u;		/* index into user[] array */

	x  = c->week_margin + c->week_daywidth + c->week_gap;
	yd = 2*c->week_margin + c->week_title + c->week_hour + c->week_gap + 2;
	ys = c->week_barheight + 2*c->week_bargap;
	xend = x + c->week_hourwidth * (c->week_maxhour - c->week_minhour);

	for (d=0; d < c->ndays_in_week; d++) {
	    for (y=yd, vert=week.tree[d]; vert; vert=vert->down, y+=ys) {
		if (yc < y || yc >= y+ys)
			continue;
		if (xc < x) {
			create_list_popup(mainlist, curr_week + d*86400,
								86400, 0, 0);
			return;
		}
		for (horz=vert; horz; horz=horz->next) {
			u = name_to_user(horz->entry->user);
			draw_bar(horz, u >= 0 ? user[u].color : 0,
				 x, y + c->week_bargap);

			ep = horz->entry;
			btime = TOD(ep->time);
			if (config.weekwarn)
				btime -= ep->early_warn > ep->late_warn
						? ep->early_warn
						: ep->late_warn;
			if (btime < c->week_minhour*3600)
				btime = c->week_minhour*3600;
			e = b = x + XPOS(TOD(btime));
			if (!ep->notime)
				e += ep->length * c->week_hourwidth/3600;
			if (e < b + MINLEN)
				e = b + MINLEN;
			if (!horz->textinside)
				e += horz->textlen;
			if (xc >= b && xc < e ||
			    xc >= xend-MINLEN && b >= xend) {
				if (time-last_time < DBLTIME && ep == last_ep){
					if (u >= 0 && user[u].readonly)
						print_button(week.info,
						 "Cannot edit, no permission");
					else
						create_list_popup(mainlist,
								0, 0, 0, ep);
					last_ep = 0;
					return;
				} else if (horz->entry->user)
					print_button(week.info, "%s %s: %s",
						mktimestring(ep->time, FALSE),
						horz->entry->user,
						ep->note ? ep->note : "");
				else
					print_button(week.info, "%s %s",
						mktimestring(ep->time, FALSE),
						ep->note ? ep->note : "");
				last_ep   = ep;
				last_time = time;
				return;
			}
		}
	    }
	    yd += week.nlines[d] * ys + c->week_gap;
	}
	print_button(week.info, " ");
}


/*
 * If the specified date is in the current week, redraw everything (something
 * changed on the specified day).
 */

draw_week_day(day, month, year)
	int		day, month, year;
{
	struct tm	tm;
	time_t		date;

	tm.tm_year = year;
	tm.tm_mon  = month;
	tm.tm_mday = day;
	tm.tm_hour = 0;
	tm.tm_min  = 0;
	tm.tm_sec  = 0;
	date = tm_to_time(&tm);
	if (date >= curr_week && date < curr_week*config.ndays_in_week*86400) {
		build_week(FALSE);
		draw_week_calendar();
	}
}


/*---------------------------------- drawing front-ends ---------------------*/
/*
 * draw the entire week menu, unconditionally. Calling create_week_menu
 * will cause a callback to redraw_week_calendar, either explicitly or
 * because of an expose event.
 */

void draw_week_calendar()
{
	if (!week.canvas)
		return;
	create_week_menu();			/* recalc & resize */
}


/*
 * draw one week into the current window, in the specified region (all
 * if null).
 */

static void gen_week(), g_setcolor(), g_fillrect();
static struct gen {
	FILE	*fp;			/* if nz, fprintf PostScript here */
	Window	window;			/* if fp is 0, draw into this window */
} gen;

void redraw_week_calendar(region)
	Region region;
{
	if (!week.canvas) {
		if (region)
			XDestroyRegion(region);
		return;
	}
	if (region)
		XSetRegion(display, gc, region);

	gen.fp     = 0;
	gen.window = XtWindow(week.canvas);
	g_setcolor(COL_WBACK);
	g_fillrect(0, 0, week.canvas_xs, week.canvas_ys);
	gen_week();

	if (region) {
		XSetClipMask(display, gc, None);
		XDestroyRegion(region);
	}
}


/*
 * print PostScript week calendar to file fp. Paper sizes are chosen so
 * everything fits on both A4 (297x210 mm) and letter (11x8.5 in) sizes.
 * 36 points (1/2 in) is reserved for a margin around all edges. Landscape
 * is done by "rotating the paper", not swapping X/Y coords.
 */

static double ps_xsize, ps_ysize;	/* PS paper size in inches */
static double ps_xoff,  ps_yoff;	/* PS paper size in screen points */
static BOOL   ps_landscape;

void print_week_calendar(fp, landscape)
	FILE	*fp;
	BOOL	landscape;
{
	time_t time = get_time();
	struct tm *tm = time_to_tm(time);
	time -= (tm->tm_wday + 6 + config.sunday_first)%7 * 86400;
	if (!curr_week)
		curr_week = time;
	if (config.pr_omit_appts)
		destroy_week();
	else
		build_week(config.pr_omit_priv);
	fprintf(fp,
"%%!PS-Adobe\n\
%%%%Creator: plan\n\
%%%%Title: Week Calendar\n\
%%%%EndComments\n\
0.5 setlinewidth\n\
/DefNormFont /Helvetica findfont def\n");
	if (ps_landscape = landscape) {
		fprintf(fp,
"newpath clippath pathbbox pop exch pop exch add 0 translate 90 rotate\n");
		ps_xsize = 10.0 * 72;
		ps_ysize =  7.2 * 72;
		ps_xoff  = week.canvas_xs;
		ps_yoff  = week.canvas_ys > 800 ? week.canvas_ys : 800;
	} else {
		ps_xsize =  7.2 * 72;
		ps_ysize = 10.0 * 72;
		ps_xoff  = week.canvas_xs;
		ps_yoff  = week.canvas_ys > 1200 ? week.canvas_ys : 1200;
	}
	gen.fp = fp;
	gen_week();
	fprintf(fp, "showpage\n");
}


/*---------------------------------- low-level drawing ----------------------*/

#define XSCALE(x) (36.0 +	     (x)  * ps_xsize / ps_xoff)
#define YSCALE(y) (36.0 + (ps_yoff - (y)) * ps_ysize / ps_yoff)

/*
 * The g_setcolor routine uses white for COL_CALTODAY, so today is not
 * highlighted.
 */

static void g_setcolor(color)
{
	if (gen.fp) {
		double col;
		if (color >= COL_WUSER_1 && color <= COL_WUSER_7)
			col = (color - COL_WUSER_1) / 7.0 * 0.15 + 0.8;
		else if (color >= COL_HBLACK && color <= COL_HWHITE)
			col = (color - COL_HBLACK) / 7.0 * 0.3;
		else switch(color) {
		  case COL_WUSER_0:	col = 1.0;	break;
		  case COL_CALTODAY:	col = 1.0;	break;
		  case COL_STD:		col = 0;	break;
		  case COL_WBACK:	col = 1.0;	break;
		  case COL_WBOXBACK:	col = 0.97;	break;
		  case COL_WDAY:	col = 0;	break;
		  case COL_WFRAME:	col = 0;	break;
		  case COL_WGRID:	col = 0;	break;
		  case COL_WNOTE:	col = 0;	break;
		  case COL_WTITLE:	col = 0;	break;
		  case COL_WWARN:	col = 0.95;	break;
		  case COL_WEEKDAY:	col = 0;	break;
		  case COL_WEEKEND:	col = 0;	break;
		  default:		col = 0;	break;
		}
		fprintf(gen.fp, "%g setgray ", col);
	} else
		set_color(color);
}

static void g_setfont(fonti)
{
	if (gen.fp) {
		fprintf(gen.fp, "DefNormFont %d scalefont setfont\n",
			ps_landscape ? fonti == FONT_WTITLE ? 22 : 9
				     : fonti == FONT_WTITLE ? 15 : 7);
	}
#ifdef JAPAN
	else if (fonti == FONT_JNOTE)
		XSetFont(display, jgc, font[fonti]->fid);
#endif
	else
		XSetFont(display, gc, font[fonti]->fid);
}

static void g_fillrect(x, y, xs, ys)
{
	if (gen.fp) {
		fprintf(gen.fp,
     "%g %g moveto %g %g lineto %g %g lineto %g %g lineto %g %g lineto fill\n",
   			XSCALE(x),    YSCALE(y),
   			XSCALE(x+xs), YSCALE(y),
			XSCALE(x+xs), YSCALE(y+ys),
			XSCALE(x),    YSCALE(y+ys),
   			XSCALE(x),    YSCALE(y));
	} else
		XFillRectangle(display, gen.window, gc, x, y, xs, ys);
}

static void g_drawtext(x, y, text, len)
	char	*text;
{
	if (gen.fp) {
		fprintf(gen.fp, "%g %g moveto (%s) show\n",
   			XSCALE(x), YSCALE(y), text);
	} else
		XDrawString(display, gen.window, gc, x, y, text, len);
}

#ifdef JAPAN
static void g_drawtext16(x, y, text, len)
	char	*text;
{
	if (gen.fp) {
		fprintf(gen.fp, "%g %g moveto (%s) show\n",
   			XSCALE(x), YSCALE(y), text);
	} else
		XDrawString16(display, gen.window, jgc, x, y, text, len);
}
#endif

static void g_drawpoly(point, num)
	XPoint	*point;
{
	if (gen.fp) {
		int n;
		fprintf(gen.fp, "%g %g moveto ",
   			XSCALE(point[0].x), YSCALE(point[0].y));
		for (n=1; n < num; n++)
			fprintf(gen.fp, "%g %g lineto ",
				XSCALE(point[n].x), YSCALE(point[n].y));
		fprintf(gen.fp, "fill\n");
	} else
		XFillPolygon(display, gen.window, gc, point, num, Convex,
							CoordModeOrigin);
}

static void g_drawlines(point, num)
	XPoint	*point;
{
	if (gen.fp) {
		int n;
		fprintf(gen.fp, "%g %g moveto ",
   			XSCALE(point[0].x), YSCALE(point[0].y));
		for (n=1; n < num; n++)
			fprintf(gen.fp, "%g %g lineto ",
				XSCALE(point[n].x), YSCALE(point[n].y));
		fprintf(gen.fp, "stroke\n");
	} else
		XDrawLines(display, gen.window, gc, point, num,
							CoordModeOrigin);
}


/*---------------------------------- high-level drawing ---------------------*/
/*
 * draw the week, using low-level roiutines
 */

static void gen_week()
{
	register struct config	*c = &config;
	char			buf[40], *p;
	XRectangle		rects[25+2];
	int			i, l, x, y;

	for (i=0; i < c->ndays_in_week; i++)		/* day background */
		draw_week_day_background(i);
							/* thick lines */
	g_setcolor(COL_WGRID);
	g_fillrect(c->week_margin,
		   2*c->week_margin + c->week_title + c->week_hour + 1,
		   week.canvas_xs - 2*c->week_margin +1,
		   2);
	g_fillrect(c->week_margin,
		   week.canvas_ys - c->week_margin +2,
		   week.canvas_xs - 2*c->week_margin +1,
		   2);
							/* thin lines */
	x = c->week_margin + c->week_daywidth + c->week_gap;
	y = 2*c->week_margin + c->week_title + c->week_hour + 3;
	for (i=0; i <= c->week_maxhour-c->week_minhour; i++)
		g_fillrect(x + i * c->week_hourwidth, y, 1,
				week.canvas_ys - c->week_margin - y + 3);

	for (i=0; i < c->ndays_in_week; i++)		/* day foreground */
		draw_week_day_foreground(i);
							/* hour labels */
	g_setcolor(COL_WDAY);
	g_setfont(FONT_WHOUR);
	for (i=0; i <= c->week_maxhour-c->week_minhour; i++) {
		p = mktimestring((i + c->week_minhour) * 3600, FALSE);
		l = strlen_in_pixels(p, FONT_WHOUR);
		g_drawtext(x + i * c->week_hourwidth - l/2,
			   2*c->week_margin + c->week_title + c->week_hour -2,
			   p, strlen(p));
	}
	strcpy(buf, mkdatestring(curr_week));		/* centered title */
	strcat(buf, " - ");
	strcat(buf, mkdatestring(curr_week + (c->ndays_in_week-1)*86400));
	g_setcolor(COL_WTITLE);
	g_setfont(FONT_WTITLE);
	g_drawtext((week.canvas_xs - strlen_in_pixels(buf,FONT_WTITLE))/2, 
		   c->week_margin + c->week_title * 3/4,
		   buf, strlen(buf));
	g_setcolor(COL_STD);
}


/*
 * draw the background of one day of the week at its position. This includes
 * the day number and the box, but not the bars themselves. The bars are
 * printed after the thin hour lines are drawn by the caller, by calling the
 * following routine.
 */

static void draw_week_day_background(wday)
	int			wday;		/* day of the week, < NDAYS */
{
	register struct config	*c = &config;
	time_t			today;		/* today's date */
	struct holiday		*hp, *shp;	/* to check for holidays */
	char			*errmsg;	/* holiday parser error */
	char			buf[20];	/* holiday text buffer */
	struct tm		*tm;		/* today's date as m/d/y */
	char			*p;		/* temp for day name */
	int			x, y;		/* upper left of day bk box */
	int			ys;		/* height of bar line w/ gaps*/
	int			yt;		/* text y position */
	int			yts;		/* vert space for text */
	int			i;

	if (week.nlines[wday] == 0)
		return;
	x  = c->week_margin + c->week_daywidth + c->week_gap;
	y  = 2*c->week_margin + c->week_title + c->week_hour + c->week_gap + 2;
	ys = c->week_barheight + 2*c->week_bargap;
	for (i=0; i < wday; i++)
		y += week.nlines[i] * ys + c->week_gap;

							/* clear weekday name*/
	yts = week.nlines[wday] * ys + c->week_gap -1;
	today = get_time() - (curr_week + wday*86400);
	g_setcolor(today < 0 || today >= 86400 ? COL_WBACK : COL_CALTODAY);
	g_fillrect(c->week_margin, y, c->week_daywidth, yts);

							/* weekday name */
	yt = y + font[FONT_WDAY]->max_bounds.ascent;
	i = (wday + 7 - c->sunday_first) % 7;
	p = mkdatestring(curr_week + wday * 86400);
	truncate_string(p, c->week_daywidth-2, FONT_WDAY);
	g_setcolor(COL_WDAY);
	g_setfont(FONT_WDAY);
	g_drawtext(c->week_margin, yt, p, strlen(p));
	yt += font[FONT_WDAY]->max_bounds.ascent +
	      font[FONT_WDAY]->max_bounds.descent;
							/* holidays */
	tm = time_to_tm(curr_week + wday*86400);
	if (errmsg = parse_holidays(tm->tm_year, FALSE))
		create_error_popup(mainmenu.cal, 0, errmsg);
	shp = &sm_holiday[tm->tm_yday];
	hp  =    &holiday[tm->tm_yday];
	for (i=0; i < 2; i++, shp=hp)
		if (shp->string && yt < y + yts) {
			strncpy(buf, shp->string, sizeof(buf)-1);
			truncate_string(buf, c->week_daywidth-2, FONT_WDAY);
			g_setcolor(shp->stringcolor ? shp->stringcolor :
				   shp->daycolor    ? shp->daycolor
						    : COL_WDAY);
			g_drawtext(c->week_margin + c->week_daywidth-2 -
					strlen_in_pixels(buf, FONT_WDAY),
				   yt, buf, strlen(buf));
			yt += font[FONT_WDAY]->max_bounds.ascent +
			      font[FONT_WDAY]->max_bounds.descent;
		}
							/* appointment box */
	g_setcolor(COL_WBOXBACK);
	g_fillrect(x, y, week.canvas_xs - c->week_margin - x,
			 week.nlines[wday] * ys);
}


/*
 * draw the foreground of one day of the week at its position. This includes
 * all the bars. The background and the thin vertical lines have already been
 * drawn by the caller.
 */

static void draw_week_day_foreground(wday)
	int			wday;		/* day of the week, < NDAYS */
{
	register struct config	*c = &config;
	struct weeknode		*vert, *horz;	/* week node scan pointers */
	int			x, y;		/* upper left of day bk box */
	int			ys;		/* height of bar line w/ gaps*/
	int			i, u;

	x  = c->week_margin + c->week_daywidth + c->week_gap;
	y  = 2*c->week_margin + c->week_title + c->week_hour + c->week_gap + 2;
	ys = c->week_barheight + 2*c->week_bargap;
	for (i=0; i < wday; i++)
		y += week.nlines[i] * ys + c->week_gap;

	for (vert=week.tree[wday]; vert; vert=vert->down, y+=ys)
		for (horz=vert; horz; horz=horz->next) {
			u = name_to_user(horz->entry->user);
			draw_bar(horz, u >= 0 ? user[u].color : 0,
				 x, y + c->week_bargap);
		}
}


/*
 * draw one entry bar.
 */

static void draw_bar(node, color, x, y)
	struct weeknode		*node;		/* entry node to print */
	int			color;		/* color, 0..7 */
	int			x, y;		/* top left pos in canvas */
{
	register struct entry	*ep = node->entry;
	register struct config	*c = &config;
	int			xend;		/* right margin of chart */
	int			w[2], b, e;	/* early/late, begin, end */
	int			ee;		/* e with min length applied */
	int			i;		/* warning counter, 0..1 */
	int			slope;		/* delta-x of the arrowheads */
	XPoint			point[7];	/* polygon vertex list */
#ifdef JAPAN
	int			ji, jdeltax, plen;
	XChar2b			jstr[50];
	strpack			partialstr[MAXPARTIALSTRING];
	unsigned char		strpool[MAXPARTIALCHAR];
#endif

	slope = c->week_barheight * SLOPE;
	xend = x + c->week_hourwidth * (c->week_maxhour - c->week_minhour);
	i = ep->early_warn > ep->late_warn;
	w[!i] = TOD(ep->time) - ep->early_warn;
	w[ i] = TOD(ep->time) - ep->late_warn;
	b     = TOD(ep->time);
	e     = TOD(ep->time) + (ep->notime ? 1200 : ep->length);
	w[0]  = x + XPOS(w[0]);
	w[1]  = x + XPOS(w[1]);
	b     = x + XPOS(b);
	e     = x + XPOS(e);
	ee    = e < b+MINLEN ? b+MINLEN : e;
	if (ep->notime && config.week_bignotime) {
		b = 0;
		e = ee = 9999;
	}
	if (config.weekwarn && !ep->notime)
	    for (i=0; i < 2; i++)
		if (w[i] < b && w[i] <= w[1] && b+slope > 0) {
			point[5].x =
			point[0].x =
			point[2].x = BOUND(w[i]+slope, x, xend);
			point[1].x = BOUND(w[i], x, xend);
			point[3].x =
			point[4].x = BOUND(b+slope, point[0].x, xend);
			point[5].y =
			point[0].y =
			point[4].y = y;
			point[1].y = y + c->week_barheight/2+1;
			point[2].y =
			point[3].y = y + (c->week_barheight&~1)+1;
			g_setcolor(COL_WWARN);
			g_drawpoly(point, 5);
			g_setcolor(COL_WFRAME);
			g_drawlines(point, 6);
		}

	point[6].x =
	point[0].x =
	point[2].x = BOUND(b+slope, x, xend);
	point[1].x = BOUND(b, x, xend-MINLEN);
	point[3].x = BOUND(ee-slope, point[0].x, xend);
	point[3].x =
	point[5].x = BOUND(point[3].x, point[1].x+MINLEN, xend);
	point[4].x = BOUND(e, point[3].x, xend);
	point[6].y =
	point[0].y =
	point[5].y = y;
	point[1].y =
	point[4].y = y + c->week_barheight/2+1;
	point[2].y =
	point[3].y = y + (c->week_barheight&~1)+1;
	if (ep->notime && !config.week_bignotime)
		point[3].x = point[5].x = point[0].x;
	g_setcolor(ep->suspended ? COL_WWARN : COL_WUSER_0 + color);
	g_drawpoly(point, 6);
	g_setcolor(COL_WFRAME);
	g_drawlines(point, 7);

	if (*node->text) {
		char buf[100];
		strcpy(buf, node->text);
#ifdef JAPAN
		partialstr->strptr = strpool;
		if ((plen = mixedstrlen_in_pixels(node->text, partialstr,
						  FONT_WNOTE, FONT_JNOTE)) >
				xend - x) {
			for (plen = ji = 0; partialstr[ji].strptr != NULL &&
						ji < MAXPARTIALSTRING; ji++) {
				if (plen + partialstr[ji].pixlen > xend - x) {
								/* Truncating*/
					partialstr[ji].length *= (double)
							(xend-x-plen)/(double)
							partialstr[ji].pixlen;
					plen = xend - x;
					if (partialstr[ji].length == 0 ||
					    partialstr[ji].asciistr == False &&
					   (partialstr[ji].length &= ~1) == 0)
						partialstr[ji].strptr = NULL;
					break;
				} else
					plen += partialstr[ji].pixlen;
			}
		}
  		if (node->textinside) {
			x = point[0].x;
			x += (point[3].x-point[0].x - plen) / 2;
			if (x + plen > xend-plen)
				x = xend-1 - plen;
		} else
			x = point[4].x + 3;
#else
		if (node->textinside) {
			int l;
			x = point[0].x;
			truncate_string(buf, xend - x, FONT_WNOTE);
			l = strlen_in_pixels(buf, FONT_WNOTE);
			x += (point[3].x-point[0].x - l) / 2;
			if (x + l > xend-1)
				x = xend-1 - l;
		} else {
			x = point[4].x + 3;
			truncate_string(buf, xend - x, FONT_WNOTE);
		}
#endif
		g_setcolor(ep->acolor ? COL_HBLACK + ep->acolor-1 : COL_WNOTE);
		g_setfont(FONT_WNOTE);
#ifdef JAPAN
		g_setfont(FONT_JNOTE);
		for (ji = 0, jdeltax = x; partialstr[ji].strptr != NULL &&
						ji < MAXPARTIALSTRING; ji++) {
			if (partialstr[ji].asciistr == True)
				g_drawtext(
					jdeltax, y + c->week_barheight/2 +
					font[FONT_WDAY]->max_bounds.ascent/2,
					partialstr[ji].strptr,
					partialstr[ji].length);
			else
				g_drawtext16(
					jdeltax, y + c->week_barheight/2 +
					font[FONT_WDAY]->max_bounds.ascent/2,
					(XChar2b *)partialstr[ji].strptr,
					partialstr[ji].length/2);
			jdeltax += partialstr[ji].pixlen;
		}
#else
		g_drawtext(x, y + c->week_barheight/2
			     		+ font[FONT_WDAY]->max_bounds.ascent/2,
			   buf, strlen(buf));
#endif
	}
}
